@@ -1,13 +1,12 @@
 # vim:fileencoding=utf-8
-# Copyright (c) 2007 gocept gmbh & co. kg
+# Copyright (c) 2007-2009 gocept gmbh & co. kg
 # See also LICENSE.txt
-# $Id$
 """References to persistent objects."""
 
 import transaction
 from persistent.dict import PersistentDict
 
-import zope.app.component.hooks
+import zope.site.hooks
 import zope.annotation.interfaces
 import zope.traversing.interfaces
 import zope.traversing.api
@@ -16,7 +15,7 @@ import gocept.reference.interfaces
 
 
 def find_name(method):
-    """Tries to determine the name of the property in the instance's 
+    """Tries to determine the name of the property in the instance's
     class.
 
     The result is cached during the runtime of the process.
@@ -36,6 +35,33 @@ def find_name(method):
     return find_name_impl
 
 
+def get_manager():
+    return zope.component.getUtility(
+        gocept.reference.interfaces.IReferenceManager)
+
+
+def get_root():
+    site = zope.site.hooks.getSite()
+    locatable = zope.traversing.interfaces.IPhysicallyLocatable(site)
+    return locatable.getRoot()
+
+
+def lookup(target_key):
+    try:
+        target = zope.traversing.api.traverse(get_root(), target_key)
+    except zope.traversing.interfaces.TraversalError:
+        raise gocept.reference.interfaces.LookupError(target_key)
+    return target
+
+
+def get_storage(instance):
+    annotations = zope.annotation.interfaces.IAnnotations(instance)
+    result = annotations.get('gocept.reference')
+    if result is None:
+        result = annotations['gocept.reference'] = PersistentDict()
+    return result
+
+
 class ReferenceBase(object):
     """A base class for specific references."""
 
@@ -46,25 +72,7 @@ class ReferenceBase(object):
     @find_name
     def __delete__(self, instance):
         self._unregister(instance)
-        del self.storage(instance)[self.__name__]
-
-    @property
-    def manager(self):
-        return zope.component.getUtility(
-            gocept.reference.interfaces.IReferenceManager)
-
-    @property
-    def root(self):
-        site = zope.app.component.hooks.getSite()
-        locatable = zope.traversing.interfaces.IPhysicallyLocatable(site)
-        return locatable.getRoot()
-
-    def storage(self, instance):
-        annotations = zope.annotation.interfaces.IAnnotations(instance)
-        result = annotations.get('gocept.reference')
-        if result is None:
-            result = annotations['gocept.reference'] = PersistentDict()
-        return result
+        del get_storage(instance)[self.__name__]
 
     def needs_registration(self, instance):
         if not self.ensure_integrity:
@@ -77,13 +85,6 @@ class ReferenceBase(object):
 
         return True
 
-    def lookup(self, target_key):
-        try:
-            target = zope.traversing.api.traverse(self.root, target_key)
-        except zope.traversing.interfaces.TraversalError:
-            raise gocept.reference.interfaces.LookupError(target_key, self)
-        return target
-
 
 class Reference(ReferenceBase):
     """A descriptor for reference properties."""
@@ -94,17 +95,17 @@ class Reference(ReferenceBase):
             return self
 
         try:
-            target_key = self.storage(instance)[self.__name__]
+            target_key = get_storage(instance)[self.__name__]
         except KeyError:
             raise AttributeError(self.__name__)
         if target_key is None:
             return None
-        return self.lookup(target_key)
+        return lookup(target_key)
 
     @find_name
     def __set__(self, instance, value):
         self._unregister(instance)
-        storage = self.storage(instance)
+        storage = get_storage(instance)
         if value is None:
             storage[self.__name__] = None
             return
@@ -117,19 +118,19 @@ class Reference(ReferenceBase):
     def _unregister(self, instance):
         if not self.needs_registration(instance):
             return
-        target_key = self.storage(instance).get(self.__name__)
+        target_key = get_storage(instance).get(self.__name__)
         if not target_key:
             return
-        self.manager.unregister_reference(target_key)
+        get_manager().unregister_reference(target_key)
 
     def _register(self, instance):
         if not self.needs_registration(instance):
             return
-        target_key = self.storage(instance)[self.__name__]
+        target_key = get_storage(instance)[self.__name__]
         try:
-            self.lookup(target_key)
+            lookup(target_key)
         except gocept.reference.interfaces.LookupError:
             # _register is called after data structures have been changed.
             transaction.doom()
             raise
-        self.manager.register_reference(target_key)
+        get_manager().register_reference(target_key)
